/*****************************************************************************
*
* Emrah Akin Sisbot
* August 10
*
* File    : sparkStruct.h
* Brief   :
*
******************************************************************************/

#ifndef SPARK_STRUCT_H
#define SPARK_STRUCT_H

/*----------------------------------------------------------------------------
  INCLUDES
-----------------------------------------------------------------------------*/

#include "sparkConst.h"

/*----------------------------------------------------------------------------
  STRUCTURES
-----------------------------------------------------------------------------*/

typedef struct STRUCT_SPARK_P3D {
  GEN_STRING128 p3dFileName;
  int enableGraphic;
  int enablePersp;
} SPARK_P3D;

typedef struct STRUCT_SPARK_SCE {
  GEN_STRING128 name;
} SPARK_SCE;

typedef struct STRUCT_SPARK_STATE {
  int posPosterAv;
  int armConfPosterAv;
  int handPosterAv;
  int ptPosterAv;
  int objectPosterAv;
  int hrp2ConfPosterAv;
  int mocapPosterAv;
  int morsePosterAv;
  int oroConnectionAv;
  int updating;
} SPARK_STATE;


//-------------------- TOOLS ---------------------------

typedef struct STRUCT_SPARK_6D_COORD{
  double x;
  double y;
  double z;
  double yaw;
  double pitch;
  double roll;
} SPARK_6D_COORD;

typedef struct STRUCT_SPARK_3D_COORD {
  double x;
  double y;
  double z;
} SPARK_3D_COORD;

//-------------------- INTERFACE ------------------------

typedef struct STRUCT_SPARK_CAM_POS {
  double xdest;
  double ydest;
  double zdest;
  double dist;
  double hrot;
  double vrot;
} SPARK_CAM_POS;

typedef struct STRUCT_SPARK_INTERFACE_PARAMS {
  GEN_BOOL saveInterface;
  GEN_BOOL floor;
  GEN_BOOL walls;
  GEN_BOOL shadows;
  GEN_BOOL tiles;
  GEN_BOOL antialiasing;
} SPARK_INTERFACE_PARAMS;

typedef struct STRUCT_SPARK_INTERFACE_AGENT_PARAMS {
  GEN_STRING64 agentName;
  GEN_BOOL showFov;
  GEN_BOOL showPoiting;
  GEN_BOOL showVisibleObjects;
} SPARK_INTERFACE_AGENT_PARAMS;

//-------------------- ENVIRONMENT -----------------------

typedef struct STRUCT_SPARK_OBJECT_PARAMS {
  double MHP_OBJECT_UPDATE_DIST;
  double MHP_OBJECT_UPDATE_ANGLE;
} SPARK_OBJECT_PARAMS;

typedef struct STRUCT_SPARK_M3D_OBJECT {
  GEN_STRING64 name;
  SPARK_6D_COORD coords;
} SPARK_M3D_OBJECT;

typedef enum ENUM_SPARK_OBJECT_MOTION {
  OBJ_NONE = 0,
  OBJ_STATIC = 1,
  OBJ_MOVING = 2
} SPARK_OBJECT_MOTION;

typedef struct STRUCT_SPARK_OBJECT_MOTION_HISTORY_ARRAY {
  SPARK_OBJECT_MOTION motion[SPARK_OBJ_STATE_MAX][SPARK_OBJECT_HISTORY_LENGTH];
  unsigned long sensorTime [SPARK_OBJ_STATE_MAX][SPARK_OBJECT_HISTORY_LENGTH];
  int length;
} SPARK_OBJECT_MOTION_HISTORY_ARRAY;

typedef struct STRUCT_SPARK_ROBOT_POSTERS {
  GEN_STRING64 armConfPostername;
  GEN_STRING64 posPostername; 
  GEN_STRING64 ptPostername;
} SPARK_ROBOT_POSTERS;

typedef enum STRUCT_SPARK_HUMAN_POSTER_TYPE_ENUM {
  USE_MOCAP  = 0,
  USE_GEST   = 1,
  USE_HUMPOS = 2,
  USE_MORSE  = 3
} SPARK_HUMAN_POSTER_TYPE_ENUM;

typedef struct STRUCT_SPARK_HUMAN_POSTER {
  SPARK_HUMAN_POSTER_TYPE_ENUM posterType;
  int unused;
  GEN_STRING64 name;
} SPARK_HUMAN_POSTER;

typedef struct STRUCT_SPARK_AGENT_HISTORY_UNIT {
  SPARK_6D_COORD head;
  SPARK_3D_COORD lhand;
  SPARK_3D_COORD rhand;
  unsigned long headSensorTime;
} SPARK_AGENT_HISTORY_UNIT;

typedef struct STRUCT_SPARK_AGENT_HISTORY {
  SPARK_AGENT_HISTORY_UNIT history[SPARK_HUMAN_HISTORY_LENGTH];
  int length;
} SPARK_AGENT_HISTORY;

typedef struct STRUCT_SPARK_AGENT_SENSOR_DATA{
  int positionAv;
  int headAv;
  SPARK_6D_COORD head;
  int rightHandAv;
  SPARK_3D_COORD rightHand;
  int leftHandAv;
  SPARK_3D_COORD leftHand;
} SPARK_AGENT_SENSOR_DATA;

typedef enum STRUCT_SPARK_HUMAN_STATE_ENUM {
  // must be the same as BT_... constants in Move3d hri_planner
  STANDING = 0,
  SITTING = 1,
  MOVING = 2,
  STANDING_TRANSPARENT = 3 // For optimistic planning
} SPARK_HUMAN_STATE_ENUM;

typedef struct STRUCT_SPARK_AGENT_POS {
  GEN_STRING64 name;
  SPARK_3D_COORD pos;
  double pan;
  double tilt;
  SPARK_HUMAN_STATE_ENUM state;
  int nouse;
} SPARK_AGENT_POS;

typedef struct STRUCT_SPARK_HUMAN_CONFIGURATION {
  int id;
  int changed;
  double dof[40];
} SPARK_HUMAN_CONFIGURATION;

typedef struct STRUCT_M3D_ROBOT {
  GEN_STRING64 name;
  double q[M3D_MAX_DOF];
  int length;
} M3D_ROBOT;

//---------------- SITUATION ASSESSMENT ------------------

typedef enum ENUM_SPARK_PLACEMENT_RELATION {
  ISIN     = 0,
  ISON     = 1,
  ISNEXTTO = 2,
  NOPLR    = 3
} SPARK_PLACEMENT_RELATION;

typedef enum ENUM_SPARK_VISIBILITY {
  INVISIBLE = 0,
  VISIBLE   = 1
} SPARK_VISIBILITY;

typedef enum ENUM_SPARK_VISIBILITY_PLACEMENT {
  OOF = 3,
  FOV = 2,
  FOA = 1
} SPARK_VISIBILITY_PLACEMENT;

typedef struct STRUCT_SPARK_SYMB_PLACEMENT {
  GEN_STRING64 object;
  SPARK_PLACEMENT_RELATION relation;
} SPARK_SYMB_PLACEMENTS;

typedef struct STRUCT_SPARK_VISIBILITY_STATE {
  GEN_STRING64 agentName;
  SPARK_VISIBILITY visibility;
  SPARK_VISIBILITY_PLACEMENT visibilityPlacement;
  int pointedAt;
  int spatialRelation;
} SPARK_VISIBILITY_STATE;

typedef struct STRUCT_SPARK_OBJECT_STATE {
  GEN_STRING64  name;
  unsigned long lastTimeSeen;
  unsigned long lastTimeStatic;
  int detected;
  int disappeared;
  SPARK_6D_COORD pos;
  SPARK_OBJECT_MOTION motion;
  SPARK_VISIBILITY_STATE agentVisibilityState[SPARK_MAX_AGENT_NB];
  int agentVisibilityStateNb;
  //SPARK_REACHABILITY robotReachability;
  //SPARK_REACHABILITY humanReachability;
  SPARK_SYMB_PLACEMENTS placements[SPARK_OBJ_STATE_MAX];
  int placementLength;
} SPARK_OBJECT_STATE;
  
typedef struct STRUCT_SPARK_VISREACH_QUERY {
  GEN_STRING64 target;
  GEN_STRING64 agent;
} SPARK_VISREACH_QUERY;

typedef struct STRUCT_SPARK_VISREACH_RESULT {
  GEN_STRING64 result1;
  GEN_STRING64 result2;
} SPARK_VISREACH_RESULT;

typedef enum ENUM_SPARK_REACHABILITY {
  UNREACHABLE = 0,
  REACHABLE = 1,
  REACHABLE_WITH_COLLISION = 2
} SPARK_REACHABILITY;

typedef struct STRUCT_SPARK_AGENT_REACHABILITY {
  GEN_STRING128 agentName;
  GEN_STRING128 objectName;
  SPARK_REACHABILITY reachability;
} SPARK_AGENT_REACHABILITY;

typedef struct STRUCT_SPARK_ALL_REACHABILITIES {
  SPARK_AGENT_REACHABILITY agentReach[SPARK_MAX_AGENT_NB];
  int agentReachNb;
} SPARK_ALL_REACHABILITIES;

//-------------------- POSTERS -----------------------

typedef struct STRUCT_SPARK_ALL_OBJECT_STATES {
  SPARK_OBJECT_STATE object[SPARK_OBJ_STATE_MAX];
  int length;
} SPARK_ALL_OBJECT_STATES;

typedef struct STRUCT_SPARK_CURRENT_ENVIRONMENT {
  GEN_STRING64 envName;
  M3D_ROBOT robot[SPARK_MAX_THINGS_NB];
  int robotNb;
  unsigned long time;
} SPARK_CURRENT_ENVIRONMENT;

#endif
